# =============================================================================
# Copyright [2013] [Kevin Carter]
# License Information :
# This software has no warranty, it is provided 'as is'. It is your
# responsibility to validate the behavior of the routines and its accuracy
# using the code provided. Consult the GNU General Public license for further
# details (see GNU General Public License).
# http://www.gnu.org/licenses/gpl.html
# =============================================================================


def delete_actions(subparser, del_args, container_args, regex):
    delaction = subparser.add_parser('delete',
                                     parents=[del_args, container_args, regex],
                                     help=('Deletes everything in a given'
                                           ' container Including the'
                                           ' container.'))
    delaction.set_defaults(delete=True)
